/**
 * 	#####################################################################################	
 * 
 * 	Custom logger module
 * 
 * 	#####################################################################################
 */

/** -------------------------------------------------------------------------------------
 * 	Definition log levels
 * 	-------------------------------------------------------------------------------------
 */
var logLevels = { 
	fatal: 		{"level": 0, "color": "95", "name": "fatal" }, 
	error: 		{"level": 1, "color": "91", "name": "error"}, 
	warn: 		{"level": 2, "color": "93", "name": "warn"},
	ok: 		{"level": 3, "color": "92", "name": "ok"}, 
	info: 		{"level": 4, "color": "97", "name": "info"},
	debug: 		{"level": 5, "color": "96", "name": "debug"},
};

/**
 * 	function: 		formatLogDate
 * 	description: 	- formats a date object in a standard output
 */	
var _formatLogDate = function(date, withoutSeconds){
	var day 	= (date.getDate() < 10) ? "0" + date.getDate() : date.getDate();
	var month 	= (date.getMonth() < 10) ? "0" + (date.getMonth() + 1) : (date.getMonth() + 1);
	var hour 	= (date.getHours() < 10) ? "0" + date.getHours() : date.getHours();
	var minute	= (date.getMinutes() < 10) ? "0" + date.getMinutes() : date.getMinutes();
	var second	= (date.getSeconds() < 10) ? "0" + date.getSeconds() : date.getSeconds();
	var s = day + "." + month + "." + date.getFullYear() + " " + hour + ":" + minute;
	s += (withoutSeconds) ? "" : ":" + second;
	return s;
};	

/**
 * function: 		sendMail
 * description: 	- send an email with log informations
 */	
var _sendLogMail = function(options, logLevel, code, msg){	
	var subject = "["+ logLevel.name.toUpperCase() +" LOG] - " + msg + " (Code: " + code + ")"; 		
	switch(options.provider){
		case "mandrill":
			//utilsMail.mandrill.send(options.from, options.to, subject, options.name, msg);
			break;
		case "sendgrid": 	
			//utilsMail.sendgrid.send(options.from, options.to, subject, options.name, msg); 
			break;	
	}
};

var _log = function(options, code, message, metadata, withmail, logLevel){
	var logStr 		= _formatLogDate(new Date()) + ' ::: ' + logLevel.name.toUpperCase() + ' ::: [' + options.module + '] ' + message + ' (Code: ' + code + ') ';		
	var mailmsg 	= _formatLogDate(new Date()) + ' ::: ' + logLevel.name.toUpperCase() + ' ::: [' + options.module + '] ' + message + ' (Code: ' + code + ') ';	
	if(metadata){
		logStr += '\n' + util.inspect(metadata);
		mailmsg += '<br>' + util.inspect(metadata);
	}		
	if(logLevel.level < 3){
		console.error('\033['+logLevel.color+'m' + logStr + '\033[m');
	}else{
		console.log('\033['+logLevel.color+'m' + logStr + '\033[m');
	}
	if(options && options.mail){
		if(typeof withmail === 'boolean' && !withmail){ return; }  
		if(typeof withmail === 'boolean' && withmail){ return _sendLogMail(options, logLevel.name, code, mailmsg); }  
		if(withmail){ return _sendLogMail(options, logLevel.name, code, mailmsg); }  		
		if(options.mail.levels === 'all'){ return _sendLogMail(options, logLevel.name, code, mailmsg); }	
		if(Array.isArray(options.mail.levels) && options.mail.levels.indexOf(logLevel.name) != -1){ return _sendLogMail(options, logLevel, code, mailmsg); }	
		if(logLevel.level < 3){ return _sendLogMail(options, logLevel, code, mailmsg); }	
	}
};

var CLogger = {
	
	/*
	 * 	options = {
	 * 		module: '<name>',
	 * 		mail: {
	 * 			levels: [],				// default: ['fatal', 'error', 'warn']; value: "all" || ['fatal', 'error', 'warn', 'ok', 'info', 'debug']
	 * 			provider: 'mandrill', 
	 * 			from: '<email adress>', 
	 * 			name: '<name>',
	 * 			to: '<email adress>',	
	 * 		}
	 * 	}
	 */
	
  	getLogger: function (options) {
   	 	return {
	  		fatal: 	function (code, message, metadata, withmail) { _log(options, code, message, metadata, withmail, logLevels.fatal); },
	  		error: 	function (code, message, metadata, withmail) { _log(options, code, message, metadata, withmail, logLevels.error); },
	  		warn: 	function (code, message, metadata, withmail) { _log(options, code, message, metadata, withmail, logLevels.warn); },
	  		ok: 	function (code, message, metadata, withmail) { _log(options, code, message, metadata, withmail, logLevels.ok); },
	  		info: 	function (code, message, metadata, withmail) { _log(options, code, message, metadata, withmail, logLevels.info); },
	  		debug: 	function (code, message, metadata, withmail) { _log(options, code, message, metadata, withmail, logLevels.debug); },
    	};
  	}	
};

module.exports = CLogger;